@@ -79,14 +79,20 @@ Additionally you can supply an extra argument to mark_event to bypass the defaul
 :developer: Amir Salihefendic ( http://amix.dk )
 :license: BSD
 """
+from builtins import range, bytes
+
+import threading
 import redis
 import calendar
+from collections import defaultdict
 from datetime import datetime, date, timedelta
 
+local_thread = threading.local()
+
+# --- Systems related
 
-#--- Systems related ----------------------------------------------
 SYSTEMS = {
-    'default': redis.Redis(host='localhost', port=6379)
+    'default': redis.StrictRedis(host='localhost', port=6379)
 }
 
 # Should hourly be tracked as default?
@@ -94,6 +100,10 @@ SYSTEMS = {
 # of memory that Redis uses (especially with huge integers)
 TRACK_HOURLY = False
 
+# Should unique events be tracked as default?
+TRACK_UNIQUE = False
+
+
 def setup_redis(name, host, port, **kw):
     """
     Setup a redis system.
@@ -101,7 +111,7 @@ def setup_redis(name, host, port, **kw):
     :param :name The name of the system
     :param :host The host of the redis installation
     :param :port The port of the redis installation
-    :param :**kw Any additional keyword arguments will be passed to `redis.Redis`.
+    :param :**kw Any additional keyword arguments will be passed to `redis.StrictRedis`.
 
     Example::
 
@@ -109,27 +119,45 @@ def setup_redis(name, host, port, **kw):
 
         mark_event('active', 1, system='stats_redis')
     """
-    SYSTEMS[name] = redis.Redis(host=host, port=port, **kw)
+    redis_client = kw.pop('redis_client', redis.StrictRedis)
+    SYSTEMS[name] = redis_client(host=host, port=port, **kw)
+
 
 def get_redis(system='default'):
     """
     Get a redis-py client instance with entry `system`.
 
-    :param :system The name of the system, extra systems can be setup via `setup_redis`
+    :param :system The name of the system, redis.StrictRedis or redis.Pipeline
+        instance, extra systems can be setup via `setup_redis`
     """
-    return SYSTEMS[system]
+    if isinstance(system, redis.StrictRedis):
+        return system
+    else:
+        return SYSTEMS[system]
 
 
-#--- Events marking and deleting ----------------------------------------------
-def mark_event(event_name, uuid, system='default', now=None, track_hourly=None):
+# --- Events marking and deleting
+
+def mark_event(event_name, uuid, system='default', now=None, track_hourly=None,
+               track_unique=None, use_pipeline=True):
     """
     Marks an event for hours, days, weeks and months.
 
     :param :event_name The name of the event, could be "active" or "new_signups"
-    :param :uuid An unique id, typically user id. The id should not be huge, read Redis documentation why (bitmaps)
-    :param :system The Redis system to use
-    :param :now Which date should be used as a reference point, default is `datetime.utcnow`
-    :param :track_hourly Should hourly stats be tracked, defaults to bitmapist.TRACK_HOURLY, but an be changed
+    :param :uuid An unique id, typically user id. The id should not be huge,
+        read Redis documentation why (bitmaps)
+    :param :system The Redis system to use (string, Redis instance, or Pipeline
+        instance).
+    :param :now Which date should be used as a reference point, default is
+        `datetime.utcnow()`
+    :param :track_hourly Should hourly stats be tracked, defaults to
+        bitmapist.TRACK_HOURLY
+    :param :track_unique Should unique stats be tracked, defaults to
+        bitmapist.TRACK_UNIQUE
+    :param :use_pipeline Boolean flag indicating if the command should use
+        pipelines or not. You may want to avoid using pipeline within the
+        command if you provide the pipeline object in `system` argument and
+        want to manage the pipe execution yourself.
 
     Examples::
 
@@ -139,8 +167,20 @@ def mark_event(event_name, uuid, system='default', now=None, track_hourly=None):
         # Mark task completed for id 252
         mark_event('tasks:completed', 252)
     """
-    if track_hourly == None:
+    _mark(event_name, uuid, system, now, track_hourly, track_unique, use_pipeline, value=1)
+
+
+def unmark_event(event_name, uuid, system='default', now=None, track_hourly=None,
+                 track_unique=None, use_pipeline=True):
+    _mark(event_name, uuid, system, now, track_hourly, track_unique, use_pipeline, value=0)
+
+
+def _mark(event_name, uuid, system='default', now=None,
+          track_hourly=None, track_unique=None, use_pipeline=True, value=1):
+    if track_hourly is None:
         track_hourly = TRACK_HOURLY
+    if track_unique is None:
+        track_unique = TRACK_UNIQUE
 
     if not now:
         now = datetime.utcnow()
@@ -148,11 +188,62 @@ def mark_event(event_name, uuid, system='default', now=None, track_hourly=None):
     obj_classes = [MonthEvents, WeekEvents, DayEvents]
     if track_hourly:
         obj_classes.append(HourEvents)
+    if track_unique:
+        obj_classes.append(UniqueEvents)
+
+    client = get_redis(system)
+    if use_pipeline:
+        client = client.pipeline()
 
-    p = get_redis(system).pipeline()
     for obj_class in obj_classes:
-        p.setbit(obj_class.from_date(event_name, now).redis_key, uuid, 1)
-    p.execute()
+        client.setbit(obj_class.from_date(event_name, now).redis_key, uuid, value)
+
+    if use_pipeline:
+        client.execute()
+
+
+def mark_unique(event_name, uuid, system='default'):
+    """
+    Mark unique event
+
+    Unique event (aka "user flag") is an event which doesn't depend on date.
+    Can be used for storing user properties, A/B testing, extra filtering, etc.
+
+    :param :event_name The name of the event, could be "active" or "new_signups"
+    :param :uuid An unique id, typically user id. The id should not be huge,
+        read Redis documentation why (bitmaps)
+    :param :system The Redis system to use (string, Redis instance, or Pipeline
+
+    Examples::
+
+        # Mark id 42 as premium
+        mark_unique('premium', 42)
+    """
+    _mark_unique(event_name, uuid, system, value=1)
+
+
+def unmark_unique(event_name, uuid, system='default'):
+    """
+    Unmark unique event
+
+    Unique event (aka "user flag") is an event which doesn't depend on date.
+    Can be used for storing user properties, A/B testing, extra filtering, etc.
+
+    :param :event_name The name of the event, could be "active" or "new_signups"
+    :param :uuid An unique id, typically user id. The id should not be huge,
+        read Redis documentation why (bitmaps)
+    :param :system The Redis system to use (string, Redis instance, or Pipeline
+
+    Examples::
+
+        # Mark id 42 as not premium anymore
+        unmark_unique('premium', 42)
+    """
+    _mark_unique(event_name, uuid, system, value=0)
+
+
+def _mark_unique(event_name, uuid, system='default', value=1):
+    get_redis(system).setbit(UniqueEvents(event_name).redis_key, uuid, value)
 
 
 def get_event_names(system='default', prefix='', batch=10000):
@@ -162,17 +253,13 @@ def get_event_names(system='default', prefix='', batch=10000):
     """
     cli = get_redis(system)
     expr = 'trackist_%s*' % prefix
-    cursor = '0'
     ret = set()
-    while True:
-        cursor, results = cli.scan(cursor, expr, batch)
-        for result in results:
-            chunks = result.split('_')
-            event_name = '_'.join(chunks[1:-1])
-            if not event_name.startswith('bitop_'):
-                ret.add(event_name)
-        if cursor == '0':
-            break
+    for result in cli.scan_iter(match=expr, count=batch):
+        result = result.decode()
+        chunks = result.split('_')
+        event_name = '_'.join(chunks[1:-1])
+        if not event_name.startswith('bitop_'):
+            ret.add(event_name)
     return list(ret)
 
 
@@ -182,7 +269,7 @@ def delete_all_events(system='default'):
     """
     cli = get_redis(system)
     keys = cli.keys('trackist_*')
-    if len(keys) > 0:
+    if keys:
         cli.delete(*keys)
 
 
@@ -192,11 +279,24 @@ def delete_temporary_bitop_keys(system='default'):
     """
     cli = get_redis(system)
     keys = cli.keys('trackist_bitop_*')
-    if len(keys) > 0:
+    if keys:
         cli.delete(*keys)
 
 
-#--- Events ----------------------------------------------
+def delete_runtime_bitop_keys():
+    """
+    Delete all BitOp keys that were created.
+    """
+    bitop_keys = _bitop_keys()
+    for system in bitop_keys:
+        if len(bitop_keys[system]) > 0:
+            cli = get_redis(system)
+            cli.delete(*bitop_keys[system])
+    bitop_keys.clear()
+
+
+# --- Events
+
 class MixinIter:
     """
     Extends with an obj.get_uuids() returning the iterator of uuids in a key
@@ -208,13 +308,14 @@ class MixinIter:
         if val is None:
             return
 
-        zero = chr(0)
+        val = bytes(val)
+
         for char_num, char in enumerate(val):
             # shortcut
-            if char == zero:
+            if char == 0:
                 continue
             # find set bits, generate smth like [1, 0, ...]
-            bits = [(ord(char) >> i) & 1 for i in xrange(7, -1, -1)]
+            bits = [(char >> i) & 1 for i in range(7, -1, -1)]
             # list of positions with ones
             set_bits = list(pos for pos, val in enumerate(bits) if val)
             # yield everything we need
@@ -226,6 +327,21 @@ class MixinIter:
             yield item
 
 
+class MixinBitOperations:
+
+    def __invert__(self):
+        return BitOpNot(self)
+
+    def __or__(self, other):
+        return BitOpOr(self, other)
+
+    def __and__(self, other):
+        return BitOpAnd(self, other)
+
+    def __xor__(self, other):
+        return BitOpXor(self, other)
+
+
 class MixinEventsMisc:
     """
     Extends with an obj.has_events_marked()
@@ -237,20 +353,12 @@ class MixinEventsMisc:
     """
     def has_events_marked(self):
         cli = get_redis(self.system)
-        return cli.get(self.redis_key) != None
+        return cli.exists(self.redis_key)
 
     def delete(self):
         cli = get_redis(self.system)
         cli.delete(self.redis_key)
 
-    def next(self):
-        """ next object in a datetime line """
-        return self.delta(value=1)
-
-    def prev(self):
-        """ prev object in a datetime line """
-        return self.delta(value=-1)
-
     def __eq__(self, other):
         other_key = getattr(other, 'redis_key', None)
         if other_key is None:
@@ -288,7 +396,38 @@ class MixinContains:
             return False
 
 
-class YearEvents(MixinIter, MixinCounts, MixinContains, MixinEventsMisc):
+class UniqueEvents(MixinIter, MixinCounts, MixinContains,
+                   MixinEventsMisc, MixinBitOperations):
+
+    @classmethod
+    def from_date(cls, event_name, dt=None, system='default'):
+        return cls(event_name, system=system)
+
+    def __init__(self, event_name, system='default'):
+        self.event_name = event_name
+        self.system = system
+        self.redis_key = _prefix_key(event_name, 'u')
+
+    def next(self):
+        return self
+
+    def prev(self):
+        return self
+
+
+class GenericPeriodEvents(MixinIter, MixinCounts, MixinContains,
+                          MixinEventsMisc, MixinBitOperations):
+
+    def next(self):
+        """ next object in a datetime line """
+        return self.delta(value=1)
+
+    def prev(self):
+        """ prev object in a datetime line """
+        return self.delta(value=-1)
+
+
+class YearEvents(GenericPeriodEvents):
     """
     Events for a year.
 
@@ -308,9 +447,9 @@ class YearEvents(MixinIter, MixinCounts, MixinContains, MixinEventsMisc):
         self.system = system
 
         months = []
-        for m in xrange(1, 13):
-            months.append( MonthEvents(event_name, self.year, m, system) )
-        or_op = BitOpOr(*months)
+        for m in range(1, 13):
+            months.append(MonthEvents(event_name, self.year, m, system))
+        or_op = BitOpOr(system, *months)
         self.redis_key = or_op.redis_key
 
     def delta(self, value):
@@ -323,7 +462,7 @@ class YearEvents(MixinIter, MixinCounts, MixinContains, MixinEventsMisc):
         return datetime(self.year, 12, 31, 23, 59, 59, 999999)
 
 
-class MonthEvents(MixinIter, MixinCounts, MixinContains, MixinEventsMisc):
+class MonthEvents(GenericPeriodEvents):
     """
     Events for a month.
 
@@ -357,7 +496,7 @@ class MonthEvents(MixinIter, MixinCounts, MixinContains, MixinEventsMisc):
         return datetime(self.year, self.month, day, 23, 59, 59, 999999)
 
 
-class WeekEvents(MixinIter, MixinCounts, MixinContains, MixinEventsMisc):
+class WeekEvents(GenericPeriodEvents):
     """
     Events for a week.
 
@@ -394,7 +533,7 @@ class WeekEvents(MixinIter, MixinCounts, MixinContains, MixinEventsMisc):
         return datetime(e.year, e.month, e.day, 23, 59, 59, 999999)
 
 
-class DayEvents(MixinIter, MixinCounts, MixinContains, MixinEventsMisc):
+class DayEvents(GenericPeriodEvents):
     """
     Events for a day.
 
@@ -428,7 +567,7 @@ class DayEvents(MixinIter, MixinCounts, MixinContains, MixinEventsMisc):
         return datetime(self.year, self.month, self.day, 23, 59, 59, 999999)
 
 
-class HourEvents(MixinIter, MixinCounts, MixinContains, MixinEventsMisc):
+class HourEvents(GenericPeriodEvents):
     """
     Events for a hour.
 
@@ -450,8 +589,8 @@ class HourEvents(MixinIter, MixinCounts, MixinContains, MixinEventsMisc):
         self.hour = not_none(hour, now.hour)
         self.system = system
         self.redis_key = _prefix_key(event_name,
-                                     '%s-%s-%s-%s' %\
-                                         (self.year, self.month, self.day, self.hour))
+                                     '%s-%s-%s-%s' %
+                                     (self.year, self.month, self.day, self.hour))
 
     def delta(self, value):
         dt = datetime(self.year, self.month, self.day, self.hour) + timedelta(hours=value)
@@ -464,14 +603,18 @@ class HourEvents(MixinIter, MixinCounts, MixinContains, MixinEventsMisc):
         return datetime(self.year, self.month, self.day, self.hour, 59, 59, 999999)
 
 
-#--- Bit operations ----------------------------------------------
-class BitOperation:
+# --- Bit operations
+
+
+class BitOperation(MixinIter, MixinContains, MixinCounts, MixinEventsMisc,
+                   MixinBitOperations):
 
     """
     Base class for bit operations (AND, OR, XOR).
 
     Please note that each bit operation creates a new key prefixed with `trackist_bitop_`.
-    These temporary keys can be deleted with `delete_temporary_bitop_keys`.
+    These temporary keys can be deleted with `delete_temporary_bitop_keys` or
+    `delete_runtime_bitop_keys`.
 
     You can even nest bit operations.
 
@@ -505,38 +648,43 @@ class BitOperation:
 
         self.redis_key = 'trackist_bitop_%s_%s' % (op_name,
                                                    '-'.join(event_redis_keys))
+        _bitop_keys()[system].add(self.redis_key)
 
         cli = get_redis(system)
         cli.bitop(op_name, self.redis_key, *event_redis_keys)
 
 
-class BitOpAnd(BitOperation, MixinIter, MixinContains, MixinCounts, MixinEventsMisc):
+class BitOpAnd(BitOperation):
 
     def __init__(self, system_or_event, *events):
         BitOperation.__init__(self, 'AND', system_or_event, *events)
 
-class BitOpOr(BitOperation, MixinIter, MixinContains, MixinCounts, MixinEventsMisc):
+
+class BitOpOr(BitOperation):
 
     def __init__(self, system_or_event, *events):
         BitOperation.__init__(self, 'OR', system_or_event, *events)
 
-class BitOpXor(BitOperation, MixinIter, MixinContains, MixinCounts, MixinEventsMisc):
+
+class BitOpXor(BitOperation):
 
     def __init__(self, system_or_event, *events):
         BitOperation.__init__(self, 'XOR', system_or_event, *events)
 
-class BitOpNot(BitOperation, MixinIter, MixinContains, MixinCounts, MixinEventsMisc):
+
+class BitOpNot(BitOperation):
 
     def __init__(self, system_or_event, *events):
         BitOperation.__init__(self, 'NOT', system_or_event, *events)
 
 
-#--- Private ----------------------------------------------
+# --- Private
+
 def _prefix_key(event_name, date):
     return 'trackist_%s_%s' % (event_name, date)
 
-#--- Helper functions ---------------------------------------
 
+# --- Helper functions
 
 def add_month(year, month, delta):
     """
@@ -546,7 +694,7 @@ def add_month(year, month, delta):
     year, month = divmod(year * 12 + month + delta, 12)
     if month == 0:
         month = 12
-        year = year -1
+        year = year - 1
     return year, month
 
 
@@ -570,3 +718,12 @@ def iso_to_gregorian(iso_year, iso_week, iso_day):
     "Gregorian calendar date for the given ISO year, week and day"
     year_start = iso_year_start(iso_year)
     return year_start + timedelta(days=iso_day-1, weeks=iso_week-1)
+
+
+def _bitop_keys():
+    """Hold created BitOp keys (per thread)"""
+    v = getattr(local_thread, 'bitop_keys', None)
+    if v is None:
+        v = defaultdict(set)
+        setattr(local_thread, 'bitop_keys', v)
+    return v
